'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// function to generates error as you see.
var errUnit = function errUnit(errMsg) {
  return new Error('\x1B[33m' + errMsg + '\x1B[0m');
};

var cbMsg = 'Invalid callback function.';
var arrOfRegMsg = 'Invalid array type';
var arrOfRegEmptyMsg = 'Empty array of objects';
var regMsg = 'Invalid regex at: ';

// gets the regex length
var regLen = function regLen(regex) {
  return regex.length;
};

// check if input is string
var isStr = function isStr(str) {
  return typeof str === 'string';
};

// check if input is regex
var isReg = function isReg(reg) {
  return (typeof reg === 'undefined' ? 'undefined' : _typeof(reg)) === 'object';
};

// const regexize = str =>
/**
  * returns info for sym key
  *
  * @param {string} dir  - directory.
  * @callback {Requester~requestCallback}
  *
  * @returns
  * @typedef {Object} data
  * @property {number} - iat, issued at
  * @property {number} - exp, expire at
  * @property {string} - key
  */
var find = function find(dir, arrayOfReg, cb) {
  // validate callback, else every error will be handled in callback error
  if (typeof cb !== 'function') throw errUnit(cbMsg);
  // validate file.
  return _fs2.default.open(dir, 'r', function (openErr, fd) {
    if (openErr) return cb(openErr);
    // validate arrayOfReg
    if (!(arrayOfReg instanceof Array)) return cb(errUnit(arrOfRegMsg));
    // validate arrayOfReg length
    var arrayLen = regLen(arrayOfReg);
    if (arrayLen === 0) return cb(errUnit(arrOfRegEmptyMsg));
    // validate the objects inside.
    for (var i = 0; i < arrayLen; i += 1) {
      // validate regMsg in findd
      if (!isReg(arrayOfReg[i]) && !isStr(arrayOfReg[i])) {
        return cb(errUnit(regMsg + arrayOfReg[i]));
      }
    }
    // Everything is good ****************************************************
    // create array of flags
    var isFlags = [];
    // init arry with default false.
    for (var _i = 0; _i < arrayLen; _i += 1) {
      isFlags[_i] = null;
    } // flags
    var isFoundAll = false;
    var readStream = _fs2.default.createReadStream(dir, {
      encoding: 'utf8'
    });
    readStream.on('error', function (readErr) {
      return cb(readErr);
    });
    readStream.on('data', function (chunk) {
      for (var _i2 = 0; _i2 < arrayLen; _i2 += 1) {
        if (isFlags[_i2] === null) isFlags[_i2] = chunk.match(arrayOfReg[_i2]);
      }
      if (!isFlags.includes(null)) isFoundAll = true;
      if (isFoundAll) {
        readStream.destroy();
        readStream.emit('end');
      }
    });
    return readStream.on('end', function () {
      return _fs2.default.close(fd, function () {
        var report = [];
        for (var _i3 = 0; _i3 < arrayLen; _i3 += 1) {
          report[_i3] = {
            isFound: isFlags[_i3] !== null,
            reg: arrayOfReg[_i3],
            match: isFlags[_i3] && isFlags[_i3][0]
          };
        }
        return cb(null, report);
      });
    });
  });
};

exports.default = find;